package org.eclipse.acceleo.mofqa.generation.templates.services;

import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentsEList;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Package;

/**
 * @author Linda Riquelme
 * TestFrameworkServices: Global settings for test generation
 */
public class TestFrameworkServices {
	
	private static final String PROJECT_ROOT_PACKAGE = "org/eclipse/mofqa/test";
	private static final String FIREFOX_DRIVER_PATH = "C:/Users/Linda/Desktop/tesis - implementacion/desarrollo/implementacion/transformacion/org.eclipse.mofqa.test/lib/geckodriver.exe";
	
	/**
	 * @return String sentences to set firefox driver for tests.
	 */
	public static String setDriverConf(){
		return "System.setProperty(\"webdriver.firefox.marionette\",\""+TestFrameworkServices.FIREFOX_DRIVER_PATH+"\");\n\t\tWebDriver selenium = new FirefoxDriver();\n";
	}
	
	/**
	 * @return String root path for SUT project
	 */
	public static String getProjectRootPkg(){
		return PROJECT_ROOT_PACKAGE;
	}
	
	/**
	 * @return String needed test framework imports in test classes
	 */
	public static String getFrameworkBasicImports(){
		String imports = "import org.testng.annotations.*;\n";
		imports += "import org.openqa.selenium.*;\n";
		imports += "import org.openqa.selenium.support.ui.*;\n";
		imports += "import org.openqa.selenium.firefox.FirefoxDriver;\n";
		imports += "import org.testng.*;\n";
		return imports;
	} 
	
	/**
	 * @param rootPkg String with name of root package for tests
	 * @return String needed SUT imports in test classes
	 */
	public static String getTestManagerImports(String rootPkg){
		String imports = "import "+ rootPkg +".MoFQA.TestManagerClasses.*;\n";
		imports += "import "+ rootPkg +".MoFQA.AcceptanceClasses.*;\n";
		imports += "import "+ rootPkg +".MoFQA.AcceptanceClasses.PageComponent.*;\n";
		imports += "import "+ rootPkg +".MoFQA.AcceptanceClasses.PageStateSetter.*;\n";
		return imports;
	} 

	/**
	 * @param aPackage 
	 * @param rootPkg String with name of root package for tests
	 * @return String needed SUT imports in test classes
	 */
	public static String getSUTImports(Package aModel, String rootPkg){
		for(Element el : aModel.allOwnedElements()){
			if(el instanceof Package){
				Package aPackage = (Package) el;
				if(UMLVerificationServices.hasStereotype((Element) aPackage, "SUT")){
					String imports = "import "+ rootPkg +"."+ UMLServices.getElementNamespace(aPackage) +".*;\n";
					return imports;
				}
			}
		}

		return "";
	} 
	
	/**
	 * @param aPackage 
	 * @param rootPkg String with name of root package for tests
	 * @return String needed SUT imports in test classes
	 */
	public static String getVOImports(Package aModel, String rootPkg){
		String imports = "";
		for(Element el : aModel.allOwnedElements()){
			if(el instanceof Package){
				Package aPackage = (Package) el;
				for (Iterator featureIterator = aPackage.getOwnedElements().iterator();
				       featureIterator.hasNext(); ){
					Element eObject = (Element)featureIterator.next();
					if (eObject instanceof Class){
						if (UMLVerificationServices.hasStereotype(eObject, "valueObject")){
							imports += "import "+ rootPkg +"."+ UMLServices.getElementNamespace((Class) eObject) +"*;\n";
						}
					}
				}
			}
		}

		return imports;
	}	
}
