package org.eclipse.acceleo.mofqa.generation.templates.services;

import java.util.List;

import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

/**
 * @author Linda
 * UMLVerificationServices: some verifications for uml models
 */
public class UMLVerificationServices {
	/**
	 * hasStereotype: find stereotype into UML class.
	 * @param obj UML Class
	 * @param stereotypeName to be found
	 * @return  true if stereotype is applied, false if not.
	 */
	public static boolean hasStereotype(Element obj, String stereotypeName) {
		List<Stereotype> stereotypes = obj.getAppliedStereotypes();
		for (Stereotype stereotype : stereotypes) {
			if (stereotype.getName().equals(stereotypeName)) {
				return true;
			}
		}
		return false;
	}
	
	
	/**
	 * testStereotype: prints all applied stereotypes for testing
	 * @param obj UML Class
	 * @return  concatenated stereotypes string.
	 */
	public static String testStereotype(Element obj) {
		List<Stereotype> stereotypes = obj.getAppliedStereotypes();
		String ret = null;
		for (Stereotype stereotype : stereotypes) {
			ret += stereotype.getName() + " - ";
		}
		return ret;
	}
	
	
	/**
	 * existsStereotype: read list of objs to find the stereotypeName
	 * @param objs list of UML Elements where stereotype will be searched
	 * @param stereotypeName to be found
	 * @return  true if stereotype exists in any element, false otherwise.
	 */
	public static Boolean existsStereotype(List<Element> objs, String stereotypeName) {
		for(Element obj : objs){
			if(hasStereotype(obj, stereotypeName))
				return true;
		}
		return false;
	}

	
	/**
	 * existsElementType: find inner element of type typeName, into pkg
	 * @param pkg UML root package
	 * @param typeName type of element to be found, specified by name
	 * @return  true if element type exists, false otherwise.
	 */
	/*public static Boolean existsElementType(Package pkg, String typeName){
		List<Element> innerEls = ((Element)pkg).getOwnedElements();
		for(Element el : innerEls){
			if (el.eClass().getInstanceTypeName().equals(typeName)){
				return true;
			}
		}
		return false;
	}*/
	

	/**
	 * canHaveMultipleValues: verify if property was set with a multiplicity>1.
	 * @param umlProperty UML Property
	 * @return  true if multiplicity>1, false if not.
	 */
	public static boolean canHaveMultipleValues(Property umlProperty){
		/*int upper = umlProperty.getUpper();
		
		if (upper > 1)
			return true;
		
		return false;*/
		
		return umlProperty.isMultivalued();
	}
	
	
	/**
	 * hasListTypeAttribute: verify if any attribute of the class is of type list.
	 * @param clazz UML Class
	 * @return  true if exists at least one multiple-value attribute, false if not.
	 */
	public static boolean hasListTypeAttribute(Class clazz){
		for(Property prop: clazz.allAttributes()){
			if(canHaveMultipleValues(prop))
				return true;
		}
		
		return false;
	}
	
	
	/**
	 * isRequiredAttribute: verify if property was marked as required or not.
	 * @param umlProperty UML Property
	 * @return  true if required, false if not.
	 */
	public static boolean isRequiredAttribute(Property umlProperty){
		return hasStereotype(umlProperty, "required");
	}
	
	
	/**
	 * hasRequiredAttribute: verify if a required attribute exists in class.
	 * @param clazz UML Class
	 * @return  true if exists at least one required attribute, false if not.
	 */
	public static boolean hasRequiredAttribute(Class clazz){
		for(Property prop: clazz.allAttributes()){
			if(isRequiredAttribute(prop))
				return true;
		}
		
		return false;
	}
	
	public static boolean isStringType(Property p){
		return (p.getType().getName() != null && p.getType().getName().equals("String")
				|| p.getType().equals("String"));
	}
}
